@@ -106,6 +106,26 @@ G_DEFINE_TYPE_WITH_CODE (EphyEmbed, ephy_embed, GTK_TYPE_SCROLLED_WINDOW,
                          G_IMPLEMENT_INTERFACE (EPHY_TYPE_COMMAND_MANAGER,
                                                 ephy_command_manager_iface_init))
 
+static void
+uri_changed_cb (WebKitWebView *web_view,
+                GParamSpec *spec,
+                EphyEmbed *embed)
+{
+  char *uri;
+  const char *current_address;
+
+  g_object_get (web_view, "uri", &uri, NULL);
+  current_address = ephy_web_view_get_address (EPHY_WEB_VIEW (web_view));
+
+  /* We need to check if we get URI notifications without going
+     through the usual load process, as this can happen when changing
+     location within a page */
+  if (g_str_equal (uri, current_address) == FALSE)
+    ephy_web_view_set_address (EPHY_WEB_VIEW (web_view), uri);
+
+  g_free (uri);
+}
+
 static void
 title_changed_cb (WebKitWebView *web_view,
                   GParamSpec *spec,
@@ -200,6 +220,11 @@ load_status_changed_cb (WebKitWebView *view,
     ephy_web_view_set_security_level (EPHY_WEB_VIEW (view), security_level);
   } else {
     EphyWebViewNetState estate = EPHY_WEB_VIEW_STATE_UNKNOWN;
+    /* FIXME: this does not work for URLs opened without typing
+       (middle click, etc). We should use get_address, but WebKit does
+       not update its URI property until LOAD_COMMITTED, so we'll get
+       NULL here. We need to either change WebKit or fetch the address
+       we are trying to load from our side. */
     const char *loading_uri = ephy_web_view_get_typed_address (EPHY_WEB_VIEW (view));
 
     if (status == WEBKIT_LOAD_PROVISIONAL) {
@@ -678,6 +703,7 @@ ephy_embed_constructed (GObject *object)
                     "signal::download-requested", G_CALLBACK (download_requested_cb), embed,
                     "signal::notify::zoom-level", G_CALLBACK (zoom_changed_cb), embed,
                     "signal::notify::title", G_CALLBACK (title_changed_cb), embed,
+                    "signal::notify::uri", G_CALLBACK (uri_changed_cb), embed,
                     NULL);
 
   embed->priv->inspector_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
