@@ -3,6 +3,7 @@
  *  Copyright © 2002  Ricardo Fernández Pascual
  *  Copyright © 2003, 2004  Marco Pesenti Gritti
  *  Copyright © 2003, 2004, 2005  Christian Persch
+ *  Copyright © 2008  Xan López
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,12 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #include "config.h"
 
-#include "ephy-icon-entry.h"
 #include "ephy-tree-model-node.h"
 #include "ephy-location-entry.h"
 #include "ephy-marshal.h"
@@ -44,11 +43,8 @@
 
 struct _EphyLocationEntryPrivate
 {
-	EphyIconEntry *icon_entry;
-	GtkWidget *icon_ebox;
-	GtkWidget *icon;
-	GtkWidget *lock_ebox;
-	GtkWidget *lock;
+	GtkWidget *entry;
+	char *lock_stock_id;
 	GdkPixbuf *favicon;
 	GdkColor secure_bg_colour;
 	GdkColor secure_fg_colour;
@@ -75,6 +71,7 @@ struct _EphyLocationEntryPrivate
 	guint secure : 1;
 	guint apply_colours : 1;
 	guint needs_reset : 1;
+	guint show_lock : 1;
 };
 
 static const GtkTargetEntry url_drag_types [] =
@@ -92,17 +89,15 @@ static void ephy_location_entry_init (EphyLocationEntry *le);
 static gboolean ephy_location_entry_reset_internal (EphyLocationEntry *, gboolean);
 
 static void textcell_data_func (GtkCellLayout *cell_layout,
-			GtkCellRenderer *cell,
-			GtkTreeModel *tree_model,
-			GtkTreeIter *iter,
-			gpointer data);
+				GtkCellRenderer *cell,
+				GtkTreeModel *tree_model,
+				GtkTreeIter *iter,
+				gpointer data);
 static void extracell_data_func (GtkCellLayout *cell_layout,
-			GtkCellRenderer *cell,
-			GtkTreeModel *tree_model,
-			GtkTreeIter *iter,
-			gpointer data);
-
-static GObjectClass *parent_class = NULL;
+				 GtkCellRenderer *cell,
+				 GtkTreeModel *tree_model,
+				 GtkTreeIter *iter,
+				 gpointer data);
 
 enum signalsEnum
 {
@@ -114,37 +109,7 @@ enum signalsEnum
 };
 static gint signals[LAST_SIGNAL] = { 0 };
 
-#define MAX_LOC_HISTORY_ITEMS 10
-#define EPHY_LOC_HISTORY_XML_ROOT "ephy_location_history"
-#define EPHY_LOC_HISTORY_XML_VERSION "0.1"
-
-GType
-ephy_location_entry_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyLocationEntryClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) ephy_location_entry_class_init,
-			NULL,
-			NULL,
-			sizeof (EphyLocationEntry),
-			0,
-			(GInstanceInitFunc) ephy_location_entry_init
-		};
-
-		type = g_type_register_static (GTK_TYPE_TOOL_ITEM,
-					       "EphyLocationEntry",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (EphyLocationEntry, ephy_location_entry, GTK_TYPE_TOOL_ITEM)
 
 static void
 ephy_location_entry_style_set (GtkWidget *widget,
@@ -158,9 +123,9 @@ ephy_location_entry_style_set (GtkWidget *widget,
 	char *theme;
 	gboolean is_a11y_theme;
 
-	if (GTK_WIDGET_CLASS (parent_class)->style_set)
+	if (GTK_WIDGET_CLASS (ephy_location_entry_parent_class)->style_set)
 	{
-		GTK_WIDGET_CLASS (parent_class)->style_set (widget, previous_style);
+		GTK_WIDGET_CLASS (ephy_location_entry_parent_class)->style_set (widget, previous_style);
 	}
 
 	title_fg_colour = widget->style->text[GTK_STATE_INSENSITIVE];
@@ -233,7 +198,7 @@ ephy_location_entry_finalize (GObject *object)
 		priv->search_terms = NULL;
 	}
 
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (ephy_location_entry_parent_class)->finalize (object);
 }
 
 static void
@@ -242,8 +207,6 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = ephy_location_entry_finalize;
 
 	widget_class->style_set = ephy_location_entry_style_set;
@@ -311,23 +274,25 @@ update_address_state (EphyLocationEntry *entry)
 	EphyLocationEntryPrivate *priv = entry->priv;
 	const char *text;
 
-	text = gtk_entry_get_text (GTK_ENTRY (priv->icon_entry->entry));
+	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
 	priv->original_address = text != NULL &&
 				 g_str_hash (text) == priv->hash;
 }
 
 static void
-update_favicon (EphyLocationEntry *entry)
+update_favicon (EphyLocationEntry *lentry)
 {
-	EphyLocationEntryPrivate *priv = entry->priv;
-	GtkImage *image = GTK_IMAGE (priv->icon);
+	EphyLocationEntryPrivate *priv = lentry->priv;
+	GtkEntry *entry = GTK_ENTRY (priv->entry);
 
 	/* Only show the favicon if the entry's text is the
 	 * address of the current page.
 	 */
 	if (priv->favicon != NULL && priv->original_address)
 	{
-		gtk_image_set_from_pixbuf (image, priv->favicon);
+		gtk_entry_set_icon_from_pixbuf (entry,
+						GTK_ENTRY_ICON_PRIMARY,
+						priv->favicon);
 	}
 	else
 	{
@@ -335,9 +300,9 @@ update_favicon (EphyLocationEntry *entry)
 		 * the page MIME type, though text/html should be good enough
 		 * most of the time. See #337140
 		 */
-		gtk_image_set_from_icon_name (image,
-					      "text-html",
-					      GTK_ICON_SIZE_MENU);
+		gtk_entry_set_icon_from_icon_name (entry,
+						   GTK_ENTRY_ICON_PRIMARY,
+						   "text-html");
 	}
 }
 
@@ -554,8 +519,8 @@ match_selected_cb (GtkEntryCompletion *completion,
 			     state == (GDK_CONTROL_MASK | GDK_SHIFT_MASK));
 
 	ephy_location_entry_set_location (entry, item, NULL);
-	gtk_im_context_reset (GTK_ENTRY (entry->priv->icon_entry->entry)->im_context);
-	g_signal_emit_by_name (priv->icon_entry->entry, "activate");
+	gtk_im_context_reset (GTK_ENTRY (entry->priv->entry)->im_context);
+	g_signal_emit_by_name (priv->entry, "activate");
 
 	g_free (item);
 
@@ -631,7 +596,7 @@ entry_clear_activate_cb (GtkMenuItem *item,
 	EphyLocationEntryPrivate *priv = entry->priv;
 
 	priv->block_update = TRUE;
-	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), "");
+	gtk_entry_set_text (GTK_ENTRY (priv->entry), "");
 	priv->block_update = FALSE;
 	priv->user_changed = TRUE;
 }
@@ -674,7 +639,7 @@ entry_populate_popup_cb (GtkEntry *entry,
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (clear_menuitem), image);
 	g_signal_connect (clear_menuitem , "activate",
 			  G_CALLBACK (entry_clear_activate_cb), lentry);
-	is_editable = gtk_editable_get_editable (GTK_EDITABLE (priv->icon_entry->entry));
+	is_editable = gtk_editable_get_editable (GTK_EDITABLE (priv->entry));
 	gtk_widget_set_sensitive (clear_menuitem, is_editable);
 	gtk_widget_show (clear_menuitem);
 
@@ -854,11 +819,19 @@ favicon_create_drag_pixmap (EphyLocationEntry *entry,
 static void
 favicon_drag_begin_cb (GtkWidget *widget,
 		       GdkDragContext *context,
-		       EphyLocationEntry *entry)
+		       EphyLocationEntry *lentry)
 {
 	GdkPixmap *pixmap;
+	GtkEntry *entry;
+	gint index;
 
-	pixmap = favicon_create_drag_pixmap (entry, widget);
+	entry = GTK_ENTRY (widget);
+	
+	index = gtk_entry_get_current_icon_drag_source (entry);
+	if (index != GTK_ENTRY_ICON_PRIMARY)
+		return;
+
+	pixmap = favicon_create_drag_pixmap (lentry, widget);
 
 	if (pixmap != NULL)
 	{
@@ -875,50 +848,50 @@ favicon_drag_data_get_cb (GtkWidget *widget,
 			  GtkSelectionData *selection_data,
 			  guint info,
 			  guint32 time,
-			  EphyLocationEntry *entry)
+			  EphyLocationEntry *lentry)
 {
+	gint index;
+	GtkEntry *entry;
+
 	g_assert (widget != NULL);
 	g_return_if_fail (context != NULL);
 
-	ephy_dnd_drag_data_get (widget, context, selection_data,
-		time, entry, each_url_get_data_binder);
+	entry = GTK_ENTRY (widget);
+
+	index = gtk_entry_get_current_icon_drag_source (entry);
+	if (index == GTK_ENTRY_ICON_PRIMARY)
+	{
+		ephy_dnd_drag_data_get (widget, context, selection_data,
+					time, lentry, each_url_get_data_binder);
+	}
 }
 
 static gboolean
-icon_button_press_event_cb (GtkWidget *ebox,
+icon_button_press_event_cb (GtkWidget *entry,
+			    GtkEntryIconPosition position,
 			    GdkEventButton *event,
-			    EphyLocationEntry *entry)
+			    EphyLocationEntry *lentry)
 {
 	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
 
 	if (event->type == GDK_BUTTON_PRESS && 
-	    event->button == 1 && 
+	    event->button == 1 &&
 	    state == 0 /* left */)
 	{
-		GtkWidget *toplevel;
-		EphyLocationEntryPrivate *priv = entry->priv;
+		if (position == GTK_ENTRY_ICON_PRIMARY)
+		{
+			GtkWidget *toplevel;
 		
-		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (entry));
-		gtk_window_set_focus (GTK_WINDOW(toplevel),
-			      	      priv->icon_entry->entry);
+			toplevel = gtk_widget_get_toplevel (GTK_WIDGET (entry));
+			gtk_window_set_focus (GTK_WINDOW (toplevel), entry);
 
-		gtk_editable_select_region (GTK_EDITABLE (priv->icon_entry->entry), 
-					    0, -1);
-
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
-static gboolean
-lock_button_press_event_cb (GtkWidget *ebox,
-			    GdkEventButton *event,
-			    EphyLocationEntry *entry)
-{
-	if (event->type == GDK_BUTTON_PRESS && event->button == 1 /* left */)
-	{
-		g_signal_emit (entry, signals[LOCK_CLICKED], 0);
+			gtk_editable_select_region (GTK_EDITABLE (entry), 
+						    0, -1);
+		}
+		else
+		{
+			g_signal_emit (lentry, signals[LOCK_CLICKED], 0);
+		}
 
 		return TRUE;
 	}
@@ -927,76 +900,57 @@ lock_button_press_event_cb (GtkWidget *ebox,
 }
 
 static void
-ephy_location_entry_construct_contents (EphyLocationEntry *entry)
+ephy_location_entry_construct_contents (EphyLocationEntry *lentry)
 {
-	EphyLocationEntryPrivate *priv = entry->priv;
-	GtkWidget *alignment;
+	EphyLocationEntryPrivate *priv = lentry->priv;
+	GtkTargetList *targetlist;
 
-	LOG ("EphyLocationEntry constructing contents %p", entry);
+	LOG ("EphyLocationEntry constructing contents %p", lentry);
 
-	alignment = gtk_alignment_new (0.0, 0.5, 1.0, 0.0);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 1, 1);
-	gtk_container_add (GTK_CONTAINER (entry), alignment);
+	priv->entry = gtk_entry_new ();
+	gtk_container_add (GTK_CONTAINER (lentry), priv->entry);
 
-	priv->icon_entry = EPHY_ICON_ENTRY (ephy_icon_entry_new ());
-	gtk_container_add (GTK_CONTAINER (alignment),
-			   GTK_WIDGET (priv->icon_entry));
+	/* Favicon */
+	targetlist = gtk_target_list_new (url_drag_types,
+					  G_N_ELEMENTS (url_drag_types));
+	gtk_entry_set_icon_drag_source (GTK_ENTRY (priv->entry),
+					GTK_ENTRY_ICON_PRIMARY,
+					targetlist,
+					GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK);
+	gtk_target_list_unref (targetlist);
 
-	priv->icon_ebox = gtk_event_box_new ();
-	gtk_container_set_border_width (GTK_CONTAINER (priv->icon_ebox), 2);
-	gtk_event_box_set_visible_window (GTK_EVENT_BOX (priv->icon_ebox), FALSE);
-	gtk_widget_add_events (priv->icon_ebox, GDK_BUTTON_PRESS_MASK |
-			      			GDK_BUTTON_RELEASE_MASK |
-						GDK_POINTER_MOTION_HINT_MASK);
-	gtk_drag_source_set (priv->icon_ebox, GDK_BUTTON1_MASK,
-			     url_drag_types, G_N_ELEMENTS (url_drag_types),
-			     GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK);
-	g_signal_connect (priv->icon_ebox, "button-press-event",
-			  G_CALLBACK (icon_button_press_event_cb), entry);
+	gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->entry),
+					 GTK_ENTRY_ICON_PRIMARY,
+					 _("Drag and drop this icon to create a link to this page"));
 
-	gtk_widget_set_tooltip_text (priv->icon_ebox,
-				     _("Drag and drop this icon to create a link to this page"));
+	gtk_entry_set_icon_activatable (GTK_ENTRY (priv->entry),
+					GTK_ENTRY_ICON_PRIMARY,
+					TRUE);
+	gtk_entry_set_icon_from_stock (GTK_ENTRY (priv->entry),
+				       GTK_ENTRY_ICON_SECONDARY,
+				       STOCK_LOCK_INSECURE);
+	gtk_entry_set_icon_activatable (GTK_ENTRY (priv->entry),
+					GTK_ENTRY_ICON_SECONDARY,
+					TRUE);
 
-	priv->icon = gtk_image_new ();
-	gtk_container_add (GTK_CONTAINER (priv->icon_ebox), priv->icon);
+	g_object_connect (priv->entry,
+			  "signal::icon-press", G_CALLBACK (icon_button_press_event_cb), lentry,
+			  "signal::populate-popup", G_CALLBACK (entry_populate_popup_cb), lentry,
+			  "signal::key-press-event", G_CALLBACK (entry_key_press_cb), lentry,
+			  "signal::changed", G_CALLBACK (editable_changed_cb), lentry,
+			  "signal::drag-motion", G_CALLBACK (entry_drag_motion_cb), lentry,
+			  "signal::drag-drop", G_CALLBACK (entry_drag_drop_cb), lentry,
+			  "signal::drag-data-get", G_CALLBACK (favicon_drag_data_get_cb), lentry,
+			  NULL);
+			  
+	g_signal_connect_after (priv->entry, "key-press-event",
+				G_CALLBACK (entry_key_press_after_cb), lentry);
+	g_signal_connect_after (priv->entry, "activate",
+				G_CALLBACK (entry_activate_after_cb), lentry);
+	g_signal_connect_after (priv->entry, "drag-begin",
+				G_CALLBACK (favicon_drag_begin_cb), lentry);
 
-	ephy_icon_entry_pack_widget (priv->icon_entry, priv->icon_ebox, TRUE);
-
-	priv->lock_ebox = gtk_event_box_new ();
-	gtk_container_set_border_width (GTK_CONTAINER (priv->lock_ebox), 2);
-	gtk_widget_add_events (priv->lock_ebox, GDK_BUTTON_PRESS_MASK);
-	gtk_event_box_set_visible_window (GTK_EVENT_BOX (priv->lock_ebox), FALSE);
-
-	priv->lock = gtk_image_new_from_stock (STOCK_LOCK_INSECURE,
-					       GTK_ICON_SIZE_MENU);
-	gtk_container_add (GTK_CONTAINER (priv->lock_ebox), priv->lock);
-
-	ephy_icon_entry_pack_widget (priv->icon_entry, priv->lock_ebox, FALSE);
-
-	g_signal_connect (priv->icon_ebox, "drag-data-get",
-			  G_CALLBACK (favicon_drag_data_get_cb), entry);
-	g_signal_connect (priv->icon_ebox, "drag-begin",
-			  G_CALLBACK (favicon_drag_begin_cb), entry);
-
-	g_signal_connect (priv->lock_ebox, "button-press-event",
-			  G_CALLBACK (lock_button_press_event_cb), entry);
-
-	g_signal_connect (priv->icon_entry->entry, "populate_popup",
-			  G_CALLBACK (entry_populate_popup_cb), entry);
-	g_signal_connect (priv->icon_entry->entry, "key-press-event",
-			  G_CALLBACK (entry_key_press_cb), entry);
-	g_signal_connect_after (priv->icon_entry->entry, "key-press-event",
-				G_CALLBACK (entry_key_press_after_cb), entry);
-	g_signal_connect_after (priv->icon_entry->entry, "activate",
-				G_CALLBACK (entry_activate_after_cb), entry);
-	g_signal_connect (priv->icon_entry->entry, "changed",
-			  G_CALLBACK (editable_changed_cb), entry);
-	g_signal_connect (priv->icon_entry->entry, "drag-motion",
-			  G_CALLBACK (entry_drag_motion_cb), entry);
-	g_signal_connect (priv->icon_entry->entry, "drag-drop",
-			  G_CALLBACK (entry_drag_drop_cb), entry);
-
-	gtk_widget_show_all (alignment);
+	gtk_widget_show (priv->entry);
 }
 
 static void
@@ -1013,6 +967,7 @@ ephy_location_entry_init (EphyLocationEntry *le)
 	p->block_update = FALSE;
 	p->saved_text = NULL;
 	p->search_terms = NULL;
+	p->show_lock = FALSE;
 	
 	ephy_location_entry_construct_contents (le);
 
@@ -1087,7 +1042,7 @@ textcell_data_func (GtkCellLayout *cell_layout,
 	{
 		ctext = g_strdup_printf ("%s\n%s", title, url);
 
-		style = gtk_widget_get_style (priv->icon);
+		style = gtk_widget_get_style (priv->entry);
 		color = style->text[GTK_STATE_INSENSITIVE];
 
 		att = pango_attr_foreground_new
@@ -1095,6 +1050,7 @@ textcell_data_func (GtkCellLayout *cell_layout,
 		att->start_index = strlen (title)+1;
 
 		pango_attr_list_insert (list, att);
+	        g_free (title);
 	}
 	else
 	{
@@ -1129,9 +1085,9 @@ textcell_data_func (GtkCellLayout *cell_layout,
 
 	}
 
-	g_object_set (G_OBJECT (cell),
-			"attributes", list,
-			NULL);
+	g_object_set (cell,
+		      "attributes", list,
+		      NULL);
 
 	g_value_init (&text, G_TYPE_STRING);
 	g_value_take_string (&text, ctext);
@@ -1140,7 +1096,6 @@ textcell_data_func (GtkCellLayout *cell_layout,
 
 	pango_attr_list_unref (list);
 
-	g_free (title);
 	g_free (url);
 }
 
@@ -1157,13 +1112,13 @@ extracell_data_func (GtkCellLayout *cell_layout,
 
 	priv = EPHY_LOCATION_ENTRY (data)->priv;
 	gtk_tree_model_get (tree_model, iter,
-			priv->extra_col, &is_bookmark,
-			-1);
+			    priv->extra_col, &is_bookmark,
+			    -1);
 
 	if (is_bookmark)
-		g_object_set (G_OBJECT (cell),
-				"stock-id", EPHY_STOCK_BOOKMARK,
-				NULL);
+		g_object_set (cell,
+			      "stock-id", EPHY_STOCK_BOOKMARK,
+			      NULL);
 
 	g_value_init (&visible, G_TYPE_BOOLEAN);
 	g_value_set_boolean (&visible, is_bookmark);
@@ -1171,21 +1126,50 @@ extracell_data_func (GtkCellLayout *cell_layout,
 	g_value_unset (&visible);
 }
 
+/**
+ * ephy_location_entry_set_match_func:
+ * @entry: an #EphyLocationEntry widget
+ * @match_func: a #GtkEntryCompletionMatchFunc
+ * @user_data: user_data to pass to the @match_func
+ * @notify: a #GDestroyNotify, like the one given to
+ * gtk_entry_completion_set_match_func
+ *
+ * Sets the match_func for the internal #GtkEntryCompletion to @match_func.
+ *
+ **/
 void
-ephy_location_entry_set_match_func (EphyLocationEntry *le, 
+ephy_location_entry_set_match_func (EphyLocationEntry *entry, 
 				GtkEntryCompletionMatchFunc match_func,
 				gpointer user_data,
 				GDestroyNotify notify)
 {
-	EphyLocationEntryPrivate *priv = le->priv;
+	EphyLocationEntryPrivate *priv = entry->priv;
 	GtkEntryCompletion *completion;
 	
-	completion = gtk_entry_get_completion (GTK_ENTRY (priv->icon_entry->entry));
+	completion = gtk_entry_get_completion (GTK_ENTRY (priv->entry));
 	gtk_entry_completion_set_match_func (completion, match_func, user_data, notify);
 }
 
+/**
+ * ephy_location_entry_set_completion:
+ * @entry: an #EphyLocationEntry widget
+ * @model: the #GtkModel for the completion
+ * @text_col: column id to access #GtkModel relevant data
+ * @action_col: column id to access #GtkModel relevant data
+ * @keywords_col: column id to access #GtkModel relevant data
+ * @relevance_col: column id to access #GtkModel relevant data
+ * @url_col: column id to access #GtkModel relevant data
+ * @extra_col: column id to access #GtkModel relevant data
+ * @favicon_col: column id to access #GtkModel relevant data
+ *
+ * Initializes @entry to have a #GtkEntryCompletion using @model as the
+ * internal #GtkModel. The *_col arguments are for internal data retrieval from
+ * @model, like when setting the text property of one of the #GtkCellRenderer
+ * of the completion.
+ *
+ **/
 void
-ephy_location_entry_set_completion (EphyLocationEntry *le,
+ephy_location_entry_set_completion (EphyLocationEntry *entry,
 				    GtkTreeModel *model,
 				    guint text_col,
 				    guint action_col,
@@ -1195,74 +1179,94 @@ ephy_location_entry_set_completion (EphyLocationEntry *le,
 				    guint extra_col,
 				    guint favicon_col)
 {
-	EphyLocationEntryPrivate *priv = le->priv;
+	EphyLocationEntryPrivate *priv = entry->priv;
 	GtkTreeModel *sort_model;
 	GtkEntryCompletion *completion;
-	GtkCellRenderer *cell, *iconcell, *extracell;
+	GtkCellRenderer *cell;
 
-	le->priv->text_col = text_col;
-	le->priv->action_col = action_col;
-	le->priv->keywords_col = keywords_col;
-	le->priv->relevance_col = relevance_col;
-	le->priv->url_col = url_col;
-	le->priv->extra_col = extra_col;
-	le->priv->favicon_col = favicon_col;
+	entry->priv->text_col = text_col;
+	entry->priv->action_col = action_col;
+	entry->priv->keywords_col = keywords_col;
+	entry->priv->relevance_col = relevance_col;
+	entry->priv->url_col = url_col;
+	entry->priv->extra_col = extra_col;
+	entry->priv->favicon_col = favicon_col;
 
 	sort_model = gtk_tree_model_sort_new_with_model (model);
 
-	g_object_unref (model);
 	gtk_tree_sortable_set_sort_column_id 
 			(GTK_TREE_SORTABLE (sort_model),
-			 le->priv->relevance_col,
+			 entry->priv->relevance_col,
 			 GTK_SORT_DESCENDING);
 
 	completion = gtk_entry_completion_new ();
 	gtk_entry_completion_set_model (completion, sort_model);
 	g_object_unref (sort_model);
 	g_signal_connect (completion, "match-selected",
-			  G_CALLBACK (match_selected_cb), le);
+			  G_CALLBACK (match_selected_cb), entry);
 	g_signal_connect_after (completion, "action-activated",
-				G_CALLBACK (action_activated_after_cb), le);
-
-	iconcell = gtk_cell_renderer_pixbuf_new ();
+				G_CALLBACK (action_activated_after_cb), entry);
 
+#if 0
+	cell = gtk_cell_renderer_pixbuf_new ();
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
-				    iconcell, FALSE);
+				    cell, FALSE);
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
-				       iconcell, "pixbuf", favicon_col);
+				       cell, "pixbuf", favicon_col);
+#endif
 
 	cell = gtk_cell_renderer_text_new ();
-	g_object_set (cell, 
-			"ellipsize", PANGO_ELLIPSIZE_END,
-			"ellipsize-set", TRUE,
-			NULL);
+	g_object_set (cell,
+		      "ellipsize", PANGO_ELLIPSIZE_END,
+		      "ellipsize-set", TRUE,
+		      NULL);
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
 				    cell, TRUE);
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
 				       cell, "text", text_col);
+        /*
+         * As the width of the entry completion is known in advance 
+         * (as big as the entry you are completing on), we can set 
+         * any fixed width (the 1 is just this random number here). 
+         * Since the height is known too, we avoid computing the actual 
+         * sizes of the cells, which takes a lot of CPU time and does
+         * not get used anyway.
+         */
+	gtk_cell_renderer_set_fixed_size (cell, 1, -1);
 	gtk_cell_renderer_text_set_fixed_height_from_font (GTK_CELL_RENDERER_TEXT (cell), 2);
 
 	gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (completion),
 					cell, textcell_data_func,
-					le,
+					entry,
 					NULL);
 
-	extracell = gtk_cell_renderer_pixbuf_new ();
+	cell = gtk_cell_renderer_pixbuf_new ();
 	gtk_cell_layout_pack_end (GTK_CELL_LAYOUT (completion),
-				    extracell, FALSE);
+				  cell, FALSE);
 	gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (completion),
-					extracell, extracell_data_func,
-					le,
-					NULL);
+					    cell, extracell_data_func,
+					    entry,
+					    NULL);
 
 	g_object_set (completion, "inline-selection", TRUE, NULL);
 	g_signal_connect (completion, "cursor-on-match",
-			  G_CALLBACK (cursor_on_match_cb), le);
+			  G_CALLBACK (cursor_on_match_cb), entry);
 
-	gtk_entry_set_completion (GTK_ENTRY (priv->icon_entry->entry), completion);
+	gtk_entry_set_completion (GTK_ENTRY (priv->entry), completion);
 	g_object_unref (completion);
 }
 
+/**
+ * ephy_location_entry_set_location:
+ * @entry: an #EphyLocationEntry widget
+ * @address: the address to be set as the current location
+ * @typed_address: the address to be shown in the location entry or NULL
+ *
+ * Sets the current address of @entry to @address, alternatively a
+ * @typed_address can be used to determine the text to be displayed in the
+ * location entry.
+ *
+ **/
 void
 ephy_location_entry_set_location (EphyLocationEntry *entry,
 				  const char *address,
@@ -1281,19 +1285,17 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	 * bug #155824. So we save the selection iff the clipboard was owned by
 	 * the location entry.
 	 */
-	if (GTK_WIDGET_REALIZED (GTK_WIDGET (priv->icon_entry)))
+	if (GTK_WIDGET_REALIZED (GTK_WIDGET (priv->entry)))
 	{
-		GtkWidget *gtkentry = priv->icon_entry->entry;
-
-		clipboard = gtk_widget_get_clipboard (gtkentry,
+		clipboard = gtk_widget_get_clipboard (priv->entry,
 						      GDK_SELECTION_PRIMARY);
 		g_return_if_fail (clipboard != NULL);
 
-		if (gtk_clipboard_get_owner (clipboard) == G_OBJECT (gtkentry) &&
-		    gtk_editable_get_selection_bounds (GTK_EDITABLE (gtkentry),
+		if (gtk_clipboard_get_owner (clipboard) == G_OBJECT (priv->entry) &&
+		    gtk_editable_get_selection_bounds (GTK_EDITABLE (priv->entry),
 	     					       &start, &end))
 		{
-			selection = gtk_editable_get_chars (GTK_EDITABLE (gtkentry),
+			selection = gtk_editable_get_chars (GTK_EDITABLE (priv->entry),
 							    start, end);
 		}
 	}
@@ -1315,7 +1317,7 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	priv->hash = g_str_hash (address);
 
 	priv->block_update = TRUE;
-	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), text);
+	gtk_entry_set_text (GTK_ENTRY (priv->entry), text);
 	priv->block_update = FALSE;
 
 	/* We need to call update_address_state() here, as the 'changed' signal
@@ -1334,6 +1336,16 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	}
 }
 
+/**
+ * ephy_location_entry_get_can_undo:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Wheter @entry can restore the displayed user modified text to the unmodified 
+ * previous text.
+ *
+ * Return value: TRUE or FALSE indicating if the text can be restored
+ *
+ **/
 gboolean
 ephy_location_entry_get_can_undo (EphyLocationEntry *entry)
 {
@@ -1342,6 +1354,16 @@ ephy_location_entry_get_can_undo (EphyLocationEntry *entry)
 	return priv->user_changed;
 }
 
+/**
+ * ephy_location_entry_get_can_redo:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Wheter @entry can restore the displayed text to the user modified version
+ * before the undo.
+ *
+ * Return value: TRUE or FALSE indicating if the text can be restored
+ *
+ **/
 gboolean
 ephy_location_entry_get_can_redo (EphyLocationEntry *entry)
 {
@@ -1350,12 +1372,22 @@ ephy_location_entry_get_can_redo (EphyLocationEntry *entry)
 	return priv->can_redo;
 }
 
+/**
+ * ephy_location_entry_get_location:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Retrieves the text displayed by the internal #GtkEntry of @entry. This is
+ * the currently displayed text, like in any #GtkEntry.
+ *
+ * Return value: the text inside the inner #GtkEntry of @entry, owned by GTK+
+ *
+ **/
 const char *
 ephy_location_entry_get_location (EphyLocationEntry *entry)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 
-	return gtk_entry_get_text (GTK_ENTRY (priv->icon_entry->entry));
+	return gtk_entry_get_text (GTK_ENTRY (priv->entry));
 }
 
 static gboolean
@@ -1369,7 +1401,7 @@ ephy_location_entry_reset_internal (EphyLocationEntry *entry,
 
 	g_signal_emit (entry, signals[GET_LOCATION], 0, &url);
 	text = url != NULL ? url : "";
-	old_text = gtk_entry_get_text (GTK_ENTRY (priv->icon_entry->entry));
+	old_text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
 	old_text = old_text != NULL ? old_text : "";
 
 	g_free (priv->saved_text);
@@ -1391,22 +1423,48 @@ ephy_location_entry_reset_internal (EphyLocationEntry *entry,
 	return retval;
 }
 
+/**
+ * ephy_location_entry_undo_reset:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Undo a previous ephy_location_entry_reset.
+ *
+ **/
 void
 ephy_location_entry_undo_reset (EphyLocationEntry *entry)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 	
-	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), priv->saved_text);
+	gtk_entry_set_text (GTK_ENTRY (priv->entry), priv->saved_text);
 	priv->can_redo = FALSE;
 	priv->user_changed = TRUE;
 }
 
+/**
+ * ephy_location_entry_reset:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Restore the @entry to the text corresponding to the current location, this
+ * does not fire the user_changed signal. This is called each time the user
+ * presses Escape while the location entry is selected.
+ *
+ * Return value: TRUE on success, FALSE otherwise
+ *
+ **/
 gboolean
 ephy_location_entry_reset (EphyLocationEntry *entry)
 {
 	return ephy_location_entry_reset_internal (entry, FALSE);
 }
 
+/**
+ * ephy_location_entry_activate:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Set focus on @entry and select the text whithin. This is called when the
+ * user hits Control+L.
+ *
+ **/
 void
 ephy_location_entry_activate (EphyLocationEntry *entry)
 {
@@ -1415,20 +1473,37 @@ ephy_location_entry_activate (EphyLocationEntry *entry)
 
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (entry));
 
-	gtk_editable_select_region (GTK_EDITABLE(priv->icon_entry->entry),
+	gtk_editable_select_region (GTK_EDITABLE (priv->entry),
 				    0, -1);
-	gtk_window_set_focus (GTK_WINDOW(toplevel),
-			      priv->icon_entry->entry);
+	gtk_window_set_focus (GTK_WINDOW (toplevel),
+			      priv->entry);
 }
 
+/**
+ * ephy_location_entry_get_entry:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Retrieve the internal #GtkEntry of the #EphyIconEntry inside @entry.
+ *
+ * Return value: a pointer to the internal #GtkEntry of @entry
+ *
+ **/
 GtkWidget *
 ephy_location_entry_get_entry (EphyLocationEntry *entry)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 
-	return priv->icon_entry->entry;
+	return priv->entry;
 }
 
+/**
+ * ephy_location_entry_set_favicon:
+ * @entry: an #EphyLocationEntry widget
+ * @pixbuf: a #GdkPixbuf to be set as the icon of the entry
+ *
+ * Sets the icon in the internal #EphyIconEntry of @entry
+ *
+ **/
 void
 ephy_location_entry_set_favicon (EphyLocationEntry *entry,
 				 GdkPixbuf *pixbuf)
@@ -1445,13 +1520,24 @@ ephy_location_entry_set_favicon (EphyLocationEntry *entry,
 	update_favicon (entry);
 }
 
+/**
+ * ephy_location_entry_set_secure:
+ * @entry: an #EphyLocationEntry widget
+ * @secure: whether the page is secure and thus the location bar should reflect
+ * that
+ *
+ * Set @entry to give visual feedback if the page is @secure. If it is, the
+ * location bar will have its background painted differently (yellow by
+ * default).
+ *
+ **/
 void
 ephy_location_entry_set_secure (EphyLocationEntry *entry,
 				gboolean secure)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 	GtkWidget *widget = GTK_WIDGET (entry);
-	GtkWidget *gentry = ephy_icon_entry_get_entry (entry->priv->icon_entry);
+	GtkWidget *gentry = priv->entry;
 
 	priv->secure = secure;
 
@@ -1470,33 +1556,76 @@ ephy_location_entry_set_secure (EphyLocationEntry *entry,
 	gtk_widget_queue_draw (widget);
 }
 
+/**
+ * ephy_location_entry_set_show_lock:
+ * @entry: an #EphyLocationEntry widget
+ * @show_lock: if @entry should show a lock icon indicating the security level
+ * of the page
+ *
+ * If @show_lock is TRUE, the location bar will show an icon reflecting the
+ * security level of the page, by default it's shown only in secure and
+ * insecure pages (insecure meaning secure pages with something broken in such
+ * security)
+ *
+ **/
 void
 ephy_location_entry_set_show_lock (EphyLocationEntry *entry,
 				   gboolean show_lock)
 {
-	EphyLocationEntryPrivate *priv = entry->priv;
+	EphyLocationEntryPrivate *priv;
 
-	g_object_set (priv->lock_ebox, "visible", show_lock, NULL);
+	g_return_if_fail (EPHY_IS_LOCATION_ENTRY (entry));
+
+	priv = entry->priv;
+
+	priv->show_lock = show_lock != FALSE;
+
+	gtk_entry_set_icon_from_stock (GTK_ENTRY (priv->entry),
+				       GTK_ENTRY_ICON_SECONDARY,
+				       show_lock ? priv->lock_stock_id : NULL);
 }
 
+/**
+ * ephy_location_entry_set_lock_stock:
+ * @entry: an #EphyLocationEntry widget
+ * @stock_id: a stock_id from GTK+ stock icons
+ *
+ * Set the lock icon to be displayed, to actually show the icon see 
+ * ephy_location_entry_set_show_lock.
+ *
+ **/
 void
 ephy_location_entry_set_lock_stock (EphyLocationEntry *entry,
 				    const char *stock_id)
 
 {
-	EphyLocationEntryPrivate *priv = entry->priv;
+	g_return_if_fail (EPHY_IS_LOCATION_ENTRY (entry));
 
-	gtk_image_set_from_stock (GTK_IMAGE (priv->lock), stock_id,
-				  GTK_ICON_SIZE_MENU);
+	entry->priv->lock_stock_id = (char*)stock_id;
+
+	if (entry->priv->show_lock)
+		gtk_entry_set_icon_from_stock (GTK_ENTRY (entry->priv->entry),
+					       GTK_ENTRY_ICON_SECONDARY,
+					       stock_id);
 }
 
+/**
+ * ephy_location_entry_set_lock_tooltip:
+ * @entry: an #EphyLocationEntry widget
+ * @tooltip: the text to be set in the tooltip for the lock icon
+ *
+ * Set the text to be displayed when hovering the lock icon of @entry.
+ *
+ **/
 void
 ephy_location_entry_set_lock_tooltip (EphyLocationEntry *entry,
 				      const char *tooltip)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 
-	gtk_widget_set_tooltip_text (priv->lock_ebox, tooltip);
+	gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->entry),
+					 GTK_ENTRY_ICON_SECONDARY,
+					 tooltip);
 }
 
 /**
