@@ -658,7 +658,7 @@ build_tab_label (EphyNotebook *nb, EphyEmbed *embed)
 
 	g_signal_connect_object (view, "notify::icon",
 				 G_CALLBACK (sync_icon), icon, 0);
-	g_signal_connect_object (view, "notify::title",
+	g_signal_connect_object (view, "notify::embed-title",
 				 G_CALLBACK (sync_label), label, 0);
 	g_signal_connect_object (view, "notify::load-status",
 				 G_CALLBACK (sync_load_status), hbox, 0);
@@ -734,11 +734,6 @@ ephy_notebook_add_tab (EphyNotebook *notebook,
 					     NULL,
 					     position);
 
-	/* FIXME gtk bug! */
-	/* FIXME: this should be fixed in gtk 2.12; check & remove this! */
-	/* The signal handler may have reordered the tabs */
-	position = gtk_notebook_page_num (gnotebook, GTK_WIDGET (embed));
-
 	if (jump_to)
 	{
 		gtk_notebook_set_current_page (gnotebook, position);
