@@ -23,24 +23,25 @@
 
 #include "ephy-session.h"
 
-#include "ephy-embed-container.h"
-#include "ephy-window.h"
-#include "ephy-shell.h"
-#include "ephy-history-window.h"
-#include "ephy-bookmarks-editor.h"
-#include "ephy-file-helpers.h"
-#include "eel-gconf-extensions.h"
-#include "ephy-prefs.h"
-#include "ephy-gui.h"
-#include "ephy-debug.h"
-#include "ephy-stock-icons.h"
-#include "ephy-glib-compat.h"
-#include "ephy-notebook.h"
 #include "eggdesktopfile.h"
 #include "eggsmclient.h"
+#include "ephy-bookmarks-editor.h"
+#include "ephy-debug.h"
+#include "ephy-embed.h"
+#include "ephy-embed-utils.h"
+#include "ephy-embed-container.h"
+#include "ephy-extension.h"
+#include "ephy-file-helpers.h"
+#include "eel-gconf-extensions.h"
+#include "ephy-gui.h"
+#include "ephy-history-window.h"
+#include "ephy-notebook.h"
+#include "ephy-prefs.h"
+#include "ephy-shell.h"
+#include "ephy-stock-icons.h"
+#include "ephy-window.h"
 
 #include <glib/gi18n.h>
-#include <gio/gio.h>
 #include <gtk/gtk.h>
 
 #include <libxml/tree.h>
@@ -375,11 +376,11 @@ session_delete (EphySession *session,
 }
 
 static void
-load_status_notify_cb (EphyEmbed *embed,
+load_status_notify_cb (EphyWebView *view,
 		       GParamSpec *pspec,
 		       EphySession *session)
 {
-	if (ephy_embed_get_load_status (embed) == FALSE)
+	if (ephy_web_view_get_load_status (view) == FALSE)
 		ephy_session_save (session, SESSION_CRASHED);
 }
 
@@ -389,7 +390,7 @@ notebook_page_added_cb (GtkWidget *notebook,
 			guint position,
 			EphySession *session)
 {
-	g_signal_connect (embed, "notify::load-status",
+	g_signal_connect (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed), "notify::load-status",
 			  G_CALLBACK (load_status_notify_cb), session);
 }
 
@@ -402,7 +403,7 @@ notebook_page_removed_cb (GtkWidget *notebook,
 	ephy_session_save (session, SESSION_CRASHED);
 
 	g_signal_handlers_disconnect_by_func
-		(embed, G_CALLBACK (load_status_notify_cb),
+		(EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed), G_CALLBACK (load_status_notify_cb),
 		 session);
 }
 
@@ -641,6 +642,7 @@ session_command_open_uris (EphySession *session,
 	{
 		const char *url = uris[i];
 		EphyNewTabFlags page_flags;
+		WebKitNetworkRequest *request;
 
 		if (url[0] == '\0')
 		{
@@ -651,13 +653,15 @@ session_command_open_uris (EphySession *session,
 			page_flags = EPHY_NEW_TAB_OPEN_PAGE;
 		}
 
+		request = webkit_network_request_new (url);
 		embed = ephy_shell_new_tab_full (shell, window,
 						 NULL /* parent tab */,
-						 url,
+						 request,
 						 flags | page_flags,
-						 EPHY_EMBED_CHROME_ALL,
+						 EPHY_WEB_VIEW_CHROME_ALL,
 						 FALSE /* is popup? */,
 						 user_time);
+		g_object_unref (request);
 
 		window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed)));
 	}
@@ -698,10 +702,10 @@ session_command_dispatch (EphySession *session)
 			{
 				ephy_shell_new_tab_full (ephy_shell_get_default (),
 							 NULL /* window */, NULL /* tab */,
-							 NULL /* URL */,
+							 NULL /* NetworkRequest */,
 							 EPHY_NEW_TAB_IN_NEW_WINDOW |
 							 EPHY_NEW_TAB_HOME_PAGE,
-							 EPHY_EMBED_CHROME_ALL,
+							 EPHY_WEB_VIEW_CHROME_ALL,
 							 FALSE /* is popup? */,
 							 cmd->user_time);
 			}
@@ -1025,17 +1029,17 @@ write_tab (xmlTextWriterPtr writer,
 	ret = xmlTextWriterStartElement (writer, (xmlChar *) "embed");
 	if (ret < 0) return ret;
 
-	address = ephy_embed_get_address (embed);
+	address = ephy_web_view_get_address (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
 	ret = xmlTextWriterWriteAttribute (writer, (xmlChar *) "url",
 					   (const xmlChar *) address);
 	if (ret < 0) return ret;
 
-	title = ephy_embed_get_title (embed);
+	title = ephy_web_view_get_title (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed));
 	ret = xmlTextWriterWriteAttribute (writer, (xmlChar *) "title",
 					   (const xmlChar *) title);
 	if (ret < 0) return ret;
 
-	if (ephy_embed_get_load_status (embed))
+	if (ephy_web_view_get_load_status (EPHY_GET_EPHY_WEB_VIEW_FROM_EMBED (embed)))
 	{
 		ret = xmlTextWriterWriteAttribute (writer,
 						   (const xmlChar *) "loading",
@@ -1263,7 +1267,87 @@ out:
 	return ret >= 0 ? TRUE : FALSE;
 }
 
-static void
+
+static void 
+confirm_before_recover (EphyWindow* window, char* url, char* title)
+{
+	
+	GtkWidget *embed; 
+	GString *html = g_string_new ("");
+	char *message = g_markup_printf_escaped
+		/* Translators: %s refers to the LSB distributor ID, for instance MandrivaLinux */
+		(_("This page was loading when the web browser closed unexpectedly. "
+		   "This might happen again if you reload the page. If it does, please report "
+		   "the problem to the %s developers."),
+		   LSB_DISTRIBUTOR);
+
+	const char *format =  
+	  "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
+	  "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"
+	  "<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"%s\" xml:lang=\"%s\">" /* language (twice) */
+	  "<head>"
+	  "<title>"
+	  "%s" /* web title */
+	  "</title>"
+	  "<style type=\"text/css\">"
+	  "div#body {"
+	  "top: 12px;"
+	  "right: 12px;"
+	  "bottom: 12px;"
+	  "left: 12px;"
+	  "overflow: auto;"
+	  "background-color:#dcdad5;"
+	  "font: message-box;"
+	  "border: 1px solid;"
+	  "background-repeat: no-repeat;"
+	  "background-position: center left;"
+	  "padding: 12px 12px 12px 72px;"
+	  "text-align:left;"
+		
+	  "h1 {"
+	  
+	  "margin: 0;"
+	  "font-size: 1.2em;"
+	  
+	  "}"
+	  "</style>"
+	
+	  "</head>"
+
+	  "<body dir=\"%s\">" /* rtl or ltr */
+	  "<div id=\"body\">"
+
+	  "<h1>"
+	  "%s" /* head of the message */
+	  "</h1>"
+
+	  "<p> %s </p>"   /* message */
+
+	  "</div>"
+	  "</body>"
+	  "</html>";
+
+	char *language = g_strdup (pango_language_to_string (gtk_get_default_language ()));
+	g_strdelimit (language, "_-@", '\0');
+
+	g_string_printf (html, format, language, language, title, 
+			 gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL ? "rtl" : "ltr", 
+			 _("Blank page"), message);
+
+  	embed  = (GtkWidget*) ephy_shell_new_tab (ephy_shell, window, NULL, NULL, 
+						  EPHY_NEW_TAB_IN_EXISTING_WINDOW | EPHY_NEW_TAB_APPEND_LAST);
+
+	/* show generated html and put the original URL in the navigation bar */
+	webkit_web_view_load_html_string (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed),
+					  html->str, 
+					  url);	
+	g_string_free (html, TRUE);
+	g_free (message);
+	g_free (language);
+}
+
+
+static void 
 parse_embed (xmlNodePtr child,
 	     EphyWindow *window,
 	     EphySession *session)
@@ -1273,7 +1357,7 @@ parse_embed (xmlNodePtr child,
 		if (strcmp ((char *) child->name, "embed") == 0)
 		{
 			xmlChar *url, *attr;
-			char *recover_url, *freeme = NULL;
+			char *recover_url;
 			gboolean was_loading;
 
 			g_return_if_fail (window != NULL);
@@ -1285,42 +1369,38 @@ parse_embed (xmlNodePtr child,
 			xmlFree (attr);
 
 			url = xmlGetProp (child, (const xmlChar *) "url");
-			if (url == NULL) continue;
+			if (url == NULL) 
+			  continue;
 
+			/* in the case that crash happens before we receive the URL from the server,
+			   this will open an about:blank tab. See http://bugzilla.gnome.org/show_bug.cgi?id=591294
+
+			   Otherwise, if the web was fully loaded, it is reloaded again.
+			
+			*/
 			if (!was_loading ||
 			    strcmp ((const char *) url, "about:blank") == 0)
 			{
 				recover_url = (char *) url;
+
+				ephy_shell_new_tab (ephy_shell, window, NULL, recover_url,
+						    EPHY_NEW_TAB_IN_EXISTING_WINDOW |
+						    EPHY_NEW_TAB_OPEN_PAGE |
+						    EPHY_NEW_TAB_APPEND_LAST);
 			}
-			else
+			else if (was_loading && url != NULL &&
+				 strcmp ((const char *) url, "about:blank") != 0)
 			{
-				xmlChar *title;
-				char *escaped_url, *escaped_title;
+				/* shows a message to the user that warns that this page was
+				   loading during crash and make Epiphany crash again,
+				   in this case we know the URL */
 
-				title = xmlGetProp (child, (const xmlChar *) "title");
-				escaped_title = g_uri_escape_string (title ? (const char*) title : _("Untitled"),
-								     NULL, TRUE);
-
-				escaped_url = g_uri_escape_string ((const char *) url,
-								   NULL, TRUE);
-				freeme = recover_url =
-					g_strconcat ("about:recover?u=",
-						     escaped_url,
-						     "&c=UTF-8&t=",
-						     escaped_title, NULL);
-
-				xmlFree (title);
-				g_free (escaped_url);
-				g_free (escaped_title);
+				xmlChar* title = xmlGetProp (child, (const xmlChar *) "title");
+			
+				confirm_before_recover (window, (char*) url, (char*) title);
 			}
 
-			ephy_shell_new_tab (ephy_shell, window, NULL, recover_url,
-					    EPHY_NEW_TAB_IN_EXISTING_WINDOW |
-					    EPHY_NEW_TAB_OPEN_PAGE |
-					    EPHY_NEW_TAB_APPEND_LAST);
-
 			xmlFree (url);
-			g_free (freeme);
 		}
 
 		child = child->next;
